      program PSGen_test_mpi
c=============================================================================c
c Calculates the phase space volume with the multichannel MC phase space
c routine 'psgen' generated with program PSGen by
c
c                            Karol Kolodziej
c                         University of Silesia
c                            Katowice, Poland
c                        e-mail: karol.kolodziej@us.edu.pl
c                        www: https://www.kk.us.edu.pl/'
c December 2022
c modified
c=============================================================================c
 
      use kinparams
      use drivekk
      use kincuts
      use psnrmtest
      use ntdisc

c      use mpi    ! for mpif90 or mpifort
      implicit none
      include 'mpif.h'    ! for mpiifort
 
      integer :: my_id, root_process, num_procs, ierr, nkin_p

      integer :: iborn,iscan0,nscan0,ie,j,ndim,ncall,neff0,
     &           i,ncalli,niter,n,iwfrst=1,iwpsn,nmx(nkin),
     &           iprind,iit

      real(dp) :: cs0,cssc(nkin),sd2,po,
     &            cs0i,var0i,sd0i,sumin,dxmc,awmax,qw,
     &            cs0i2,cs02,csnum,csden,csacc,sdacc,chi2,chi2s,
     &            arg,val,sval,diter,th_min,xmx(nkin),
     &            partial_cs,cs0_p,sd2_p,cs0i_p,var0i_p,
     &            sd0i_p,crmaxtot,crstrd

      integer :: status(MPI_STATUS_SIZE),icall_p,ntd0_p,ntd1_p,ntd2_p,
     &           start_row,end_row,an_id,n_rows,icall_it,icall_scan,
     &           ntd0_it,ntd1_it,ntd2_it,netd0,netd1,netd2,
     &           nofr_p,nofr_t

      character(80) :: ind,ind1,ind2,ind3,indj,nmb,nml

      real(dp) :: cs_sect

      external cs_sect

c Please supply the necessary information below by giving values to
c the parameters and variables

      integer, parameter :: ne=2  ! No of the cms energies

      real(dp), dimension(ne) :: aecm=(/ ! The cms energies in GeV:
     &         5.d2,1.d3/)

      real(dp), dimension(ne) :: acsacc,asdacc
      integer, dimension(ne) :: acall,anmx
c Extra characteistic of the output file:

      character(80) :: outchr='', prcsnm, nicknm, prog_vrsn

      include 'process_nm.dat'

c     Let process 0 be the root process.

      root_process = 0

c     Now replicate this process to create parallel processes.

      call MPI_INIT (ierr)

c     Find out MY process ID, and how many processes were started.

      call MPI_COMM_RANK (MPI_COMM_WORLD, my_id, ierr)
      call MPI_COMM_SIZE (MPI_COMM_WORLD, num_procs, ierr)

c Calculate the Born cross section, iborn = 1(yes)/else(no), 
c in 'niter' iterations wioth 'ncalli' calls per iteration, i.e. 
c with 'niter x ncalli' calls;
c po is the fraction of old integration weights which should be
c used in the next iterration, recomended value from 0.d0 to 0.5d0
c If the value of a call to integrand exceeds the maximum found for 
c a given kinematical channel by 'mxfctr' then the call is discarded

      iborn=1
      niter=10
      ncalli=200000
      po=0.d0
      mxfctr=1.d3

c Scan the Born cross section with 'nscan0' calls: iscan0 = 1(yes)/else(no)?

      iscan0=1
      nscan0=100

c Select the random number generator, 
c irand=1(intrinsic Fortran generator)/2(RANLUX)

      irand=2

c Print results computed by individual processes, iprind=1(yes)/else(no)?

      iprind=1

c Perform test of the phase space normalization, ipsnrm=1(yes)/else(no)?

      ipsnrm=0

c Specify the relative accuracy 'psnac', for which phase space normalization dlips 
c recalculated from the four momenta generated with a given kinematics ikin 
c should be replaced with the phase space normalization dlips0 calculated when 
c channel ikin was called to generate the four momenta

      psnac=1.d-3

c Take into account s-channel, t-channel or g-channel kinematics only, 
c ik_sch, or ik_tch, or ik_gch =1(yes)/else(no)? Only one of them can be
c equal 1 at a time.

      ik_sch=0
      ik_tch=0
      ik_gch=0

c Perform tests of four momenta generated by the kinematical routines
c check if the four moenta are on mass shell, im_test=1(yes)/else(no)
      im_test=0
      dm_test=1.d-5
c check if the four momenta are conserved, ip_test=1(yes)/else(no)
      ip_test=0
      dp_test=1.d-11

c Impose cuts, icuts=1(yes)/0(no)

      icuts=1

c If icuts=1 then select the cuts to be imposed in file 'define_cuts.f'

c Discard kinematical channels which contribute less than qw of the dominant 
c channel in the initial scan

      qw=0.d0

c Discard kinematical channels where the inacurate phase space normalization
c is detected during the scan, iwpsn=1(yes)/else(no)

      iwpsn=0

c----------------------------------------------------------------------c
c End of input
c----------------------------------------------------------------------c

c Modify the short name of the proces:

      if (imtth == 1) then
       write(ind,'(f4.0)') mbbh
       nicknm=trim(adjustl(nicknm))//'_mbbh'//trim(adjustl(ind))
      end if

      if (iqprec == 1) then                            ! added on 29 Aug, 2019
       nicknm=trim(nicknm)//'_qp'
      end if

      if (irand == 2) then
       nicknm=trim(nicknm)//'_RANLUX'
      end if

      nicknm=trim(nicknm)//trim(outchr)

      if (ik_sch == 1) then
       nicknm=trim(nicknm)//'_sch'
      else if (ik_tch == 1) then
       nicknm=trim(nicknm)//'_tch'
      else if (ik_gch == 1) then
       nicknm=trim(nicknm)//'_gch'
      end if

      write(ind,'(i16)') my_id
      open(unit=6,file='tot_'//trim(adjustl(ind))//'_'
     & //trim(adjustl(nicknm)),status='replace',action='write')
      open(unit=7,file='memos_'//trim(adjustl(ind))//'_'
     & //trim(adjustl(nicknm)),status='replace',action='write')

c Initialize integation weights and some other parameters and print the
c corresponding information

      call param_trans(6)

      mthrs=zero
      sumin=zero

      do j=nexp-nfsp+1,nexp
       mthrs=mthrs+m(j)
      end do
      do j=1,nexp-nfsp
       sumin=sumin+m(j)
      end do

c The energy loop:

      ncalli=ncalli/num_procs

      do ie=1,ne

      ecm=aecm(ie)     !  The c.m.s. energy

      aw=1.d0/nkin
      iwkin=0
      aw_w=0.d0

      if (ecm < sumin) then 
       write(6,*)'carlocom.f: The CMS energy below the mass of initial',
     &           ' state particles'
       write(6,*) sumin,' GeV'
       exit
      end if
      if (ecm < mthrs) then 
       write(6,*)'carlocom.f: The CMS energy below the threshold',mthrs,
     &           ' GeV'
       exit
      end if

      write(6,4000)
 4000 format('#-------------------------------------------------------',
     &       '----------------------#')
      write(6,1006) ecm
      write(6,4000)
 1006 format('#                       E_cm = ',f11.3,' GeV')

      if (iborn == 1) then
       write(6,'(a,i2,a)') '# The Born cross section with full ',
     &    nfsp,' particle kinematics:'

       ndim=3*nfsp-4

       iscan=iscan0

       if (iscan == 1) then

c Scan the Born cross section in order to find the dominant channels:

        ncall=nscan0
        cs0_p=zero
        icall=0
        cssc=zero
        xmx=0.d0
        nmx=0
        ntd1=0
        ntd2=0
        ntd0=0

        write(6,5110)
 5110   format('# Results of the initial scan:')
        write(6,5120)
 5120   format('#     Kinematics       Weight         Integral      ',
     &       'St.dev.       No of calls')

        sd2_p=zero
        nkin_p=nkin/num_procs+1

        do ikin=my_id*nkin_p+1,(my_id+1)*nkin_p
         if (ikin > nkin) exit
         if (aw(ikin) > 0.d0) then
          neff0=icall
          call carlos(cs_sect,ndim,nkin,xmc,po,xmx,nmx,ncall,cs0i,
     &                                                      var0i,sd0i)
          write(6,3002) ikin,aw(ikin),aw(ikin)*cs0i,aw(ikin)*sd0i,
     &                                                icall-neff0
 3002     format('#  ',i10,'     ',e15.6,e15.6,e12.3,i10)
          cs0_p=cs0_p+aw(ikin)*cs0i
          cssc(ikin)=aw(ikin)*cs0i
          sd2_p=sd2_p+(aw(ikin)*sd0i)**2
         end if
        end do

        if (iprind == 1) then
         write(6,'(a,i4,a,e13.6,a,e9.2,a,i9,a)') '# Process', my_id, 
     &    ' computes:', cs0_p,' +/-',sqrt(sd2_p),' with', icall, 
     &    ' calls'
        end if

        icall_p=icall
        ntd0_p=ntd0
        ntd1_p=ntd1
        ntd2_p=ntd2

        call MPI_REDUCE (cs0_p, cs0, 1, MPI_DOUBLE_PRECISION,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (sd2_p, sd2, 1, MPI_DOUBLE_PRECISION,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (icall_p, icall_scan, 1, MPI_INTEGER,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (ntd0_p, ntd0, 1, MPI_INTEGER,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (ntd1_p, ntd1, 1, MPI_INTEGER,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (ntd2_p, ntd2, 1, MPI_INTEGER,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)

        call MPI_BCAST (cs0, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (sd2, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (icall_scan, 1, MPI_INTEGER, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (ntd0, 1, MPI_INTEGER, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (ntd1, 1, MPI_INTEGER, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (ntd2, 1, MPI_INTEGER, root_process, 
     &        MPI_COMM_WORLD, ierr)

c        write(6,*) 'The result is:', cs0,'+/-',sqrt(sd2),
c     &   ' with', icall_scan, ' calls'

        if (iwpsn == 1) then        ! 15 July, 2016
         do i=1,nkin
          if (iwkin(i) > 0) then
           cs0=cs0-cssc(i)
           cssc(i)=0.d0
           aw(i)=0.d0
c           iwkin(i)=0
           write(6,'(a,i7,a)')'# Contribution of kinematics',i,
     &        ' cancelled'
          end if
         end do
        end if

        write(6,4000)
        write(6,3800) icall_scan,cs0,sqrt(sd2)

 3800   format('# Born scanned with',i8,' calls = ',e15.8,'+/-',e10.3)

        if (cs0 == 0.d0) exit

 3810   format('# The maximum value of the cross section = ',e15.8)

        write(6,'(a)')'# MC calls discarded because of some '//
     &   'kinematical variables being out of their'
        write(6,'(a,i12)')'# physical ranges:',ntd0
        if (nkinst > nkins) then
         write(6,'(a,i9,i9)')'# Discarded calls in t-channel '//
     &     'kinems, for it1=1, it2=1:',ntd1,ntd2
        end if
        write(6,4000)

        do an_id = 1, num_procs-1
         start_row=an_id*nkin_p+1
         end_row=(an_id+1)*nkin_p
         if (an_id == num_procs-1) end_row=nkin
         n_rows=end_row-start_row+1
         call MPI_BCAST (n_rows, 1, MPI_INTEGER, an_id, 
     &        MPI_COMM_WORLD, ierr)
         call MPI_BCAST (cssc(start_row), n_rows, MPI_DOUBLE_PRECISION,
     &                  an_id, MPI_COMM_WORLD, ierr)
         call MPI_BCAST (aw(start_row), n_rows, MPI_DOUBLE_PRECISION,
     &                  an_id, MPI_COMM_WORLD, ierr)
         call MPI_BCAST (xmc(start_row), n_rows, MPI_DOUBLE_PRECISION,
     &                  an_id, MPI_COMM_WORLD, ierr)
         call MPI_BCAST (xmx(start_row), n_rows, MPI_DOUBLE_PRECISION,
     &                  an_id, MPI_COMM_WORLD, ierr)
         call MPI_BCAST (nmx(start_row), n_rows, MPI_INTEGER,
     &                  an_id, MPI_COMM_WORLD, ierr)
        end do

        call MPI_BCAST (cssc(1), nkin, MPI_DOUBLE_PRECISION,
     &                  root_process, MPI_COMM_WORLD, ierr)
        call MPI_BCAST (aw(1), nkin, MPI_DOUBLE_PRECISION,
     &                  root_process, MPI_COMM_WORLD, ierr)
        call MPI_BCAST (xmc(1), nkin, MPI_DOUBLE_PRECISION,
     &                  root_process, MPI_COMM_WORLD, ierr)
        call MPI_BCAST (xmx(1), nkin, MPI_DOUBLE_PRECISION,
     &                  root_process, MPI_COMM_WORLD, ierr)
        call MPI_BCAST (nmx(1), nkin, MPI_INTEGER,
     &                  root_process, MPI_COMM_WORLD, ierr)

        iscan=0

        if (qw > 0.d0) then
         write(6,'(a,e9.2,a)') '# Kinematical channels contributing '
     &     //'less than',qw,' of the dominant channel'
         write(6,'(a)') '# discarded.'
        end if

c Initialize integration weights:

        neff0=0
        xmc(0)=zero
        dxmc=zero
        awmax=zero
        do ikin=1,nkin
         if (ikin > nkin) exit
         if (aw(ikin) > zero) then
          aw(ikin)=cssc(ikin)/cs0
          if (aw(ikin) > awmax) awmax=aw(ikin)
         end if
        end do
        do ikin=1,nkin
         if (ikin > nkin) exit
         if (aw(ikin) > zero) then
          if (aw(ikin) < qw*awmax) then
           dxmc=dxmc+aw(ikin)
           aw(ikin)=zero
          end if
         end if
         xmc(ikin)=xmc(ikin-1)+aw(ikin)
        end do

        xmc=xmc/(1.d0-dxmc)

       else ! iscan
c Initialize integration weights:
        neff0=0
        xmc(0)=zero
        do ikin=1,nkin
         xmc(ikin)=xmc(ikin-1)+aw(ikin)
        end do
       end if ! iscan

c The Born cross section with standard deviation:

       cs0=0.d0
       icall=0

       write(6,3200)
 3200 format('# Calculating the Born cross section with the weights:')

       csnum=0.d0
       csden=0.d0
       cs02=0.d0
       chi2s=0.d0
       ntd1=0
       ntd2=0
       ntd0=0
       netd1=0
       netd2=0
       netd0=0
       crmax=0.d0

       do iit=1,niter
        write(6,4000)
        write(6,'(a,i3,a)') '# Iteration',iit,': channel, weight,'//
     &  ' sum, max. cs value, discarded calls'
        do ikin=1,nkin
         if (aw(ikin) > 0.d0) then
          write(6,'(a,i10,3e14.6,i6)') '#',ikin,aw(ikin),xmc(ikin),
     &                                     xmx(ikin),nmx(ikin)
         end if
        end do

        call carlos(cs_sect,ndim,nkin,xmc,po,xmx,nmx,ncalli,cs0i_p,
     &                                                 var0i_p,sd0i_p)
        if (iprind == 1) then
         write(6,'(a,i4,a,e13.6,a,e9.2,a,i9,a)') '# Process', my_id, 
     &    ' computes:', cs0i_p,' +/-',sd0i_p,' with', icall-neff0, 
     &     ' calls'
        end if

        do ikin=1,nkin
         aw(ikin)=xmc(ikin)-xmc(ikin-1)
        end do


        icall_p=icall-neff0
        ntd0_p=ntd0-netd0
        ntd1_p=ntd1-netd1
        ntd2_p=ntd2-netd2

        call MPI_REDUCE (cs0i_p,cs0i, 1, MPI_DOUBLE_PRECISION,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (var0i_p,var0i, 1, MPI_DOUBLE_PRECISION,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (icall_p, icall_it, 1, MPI_INTEGER,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (ntd0_p, ntd0_it, 1, MPI_INTEGER,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (ntd1_p, ntd1_it, 1, MPI_INTEGER,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)
        call MPI_REDUCE (ntd2_p, ntd2_it, 1, MPI_INTEGER,
     &       MPI_SUM, root_process, MPI_COMM_WORLD, ierr)

        cs0i=cs0i/num_procs
        if (num_procs > 1) then
         var0i=var0i/num_procs/(num_procs-1)
        end if
        sd0i=sqrt(var0i)

        cs0i2=cs0i*cs0i
        cs02=cs02+cs0i2
        if (cs0i > 0.d0) then
         csnum=csnum+cs0i*cs0i2/var0i
         csden=csden+cs0i2/var0i
         cs0=cs0+cs0i/niter
         chi2s=chi2s+cs0i2*(1.d0+1.d0/var0i)
        end if
        if (niter > 1) then
         sd2=sd2+(sd0i)**2/(niter*(niter-1))
        else
         sd2=(sd0i)**2
        end if

        if (csnum > 0.d0) then
         csacc=csnum/csden
         sdacc=csden*iit/cs02
         chi2=sdacc*(chi2s/csden-csacc*csacc)/(iit-0.999d0)
         sdacc=sqrt(1.d0/abs(sdacc))
        else
         csacc=0.d0
         sdacc=0.d0
         chi2=0.d0
         sdacc=0.d0
        end if

        call MPI_BCAST (csacc, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (sdacc, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (cs0i, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (var0i, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (sd0i, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (cs0, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (sd2, 1, MPI_DOUBLE_PRECISION, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (icall_it, 1, MPI_INTEGER, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (ntd0_it, 1, MPI_INTEGER, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (ntd1_it, 1, MPI_INTEGER, root_process, 
     &        MPI_COMM_WORLD, ierr)
        call MPI_BCAST (ntd2_it, 1, MPI_INTEGER, root_process, 
     &        MPI_COMM_WORLD, ierr)

        icall=neff0+icall_it
        ntd0=netd0+ntd0_it
        ntd1=netd1+ntd1_it
        ntd2=netd2+ntd2_it

        write(6,4000)
        write(6,'(a,i3,a)') '# Iteration No. ',iit,':'
        write(6,2987) cs0i,sd0i,icall_it
        write(6,'(a,e13.6,a,e9.2,a,i11,a)') '# Accumulated result: ',
     &     csacc,' +/- ',sdacc,' with ',icall,' calls'
        if (iit == niter) then
         acsacc(ie)=csacc
         asdacc(ie)=sdacc
         acall(ie)=icall
        end if
        neff0=icall
        netd0=ntd0
        netd1=ntd1
        netd2=ntd2
       end do

       write(6,4000)
       write(6,'(a)') '# Arithmetic mean result:'

       write(6,2987) cs0,sqrt(sd2),neff0
       write(6,'(a)')'# MC calls discarded because of some '//
     &   'kinematical variables being out of their'
        write(6,'(a,i12)')'# physical ranges:',ntd0

       if (nkinst > nkins) then
        write(6,'(a,i9,i9)')'# Discarded calls in t-channel '//
     &     'kinems, for it1=1 and it2=1:',ntd1,ntd2
       end if

 2987  format('# Integral =',e13.6,' +/- ',e9.2,' with ',i9,' calls')

      end if ! (iborn == 1)

      anmx(ie)=sum(nmx)+ntd0

      end do                    ! ie

c      close(10)

      write(6,4000)

      write(6,'(a)') '#   E_cm         cs    +/-   sdev     with  '
     &              //'calls      discarded calls^*'

      do ie=1,ne
       write(6,'(e11.4,e13.6,e13.6,i12,i16)') 1.d3*aecm(ie),
     &                         acsacc(ie),asdacc(ie),acall(ie),anmx(ie)
      end do
      write(6,4000)
      write(6,'(a,f10.0,a)')'#*Calls which contribute more than',
     &      mxfctr,' x max. differential cross section'
      write(6,'(a)')'# in a kinematical channel + those for which'
     & //' some kinematical variables are out'
      write(6,'(a)')'# of their physical ranges'

      close(6)

      do i=1,nkin           !  14 July, 2016
       if (iwkin(i) > 0) then
        if (iwfrst == 1) then
         write(7,'(a)')'Inacurate phase space normalization detected'//
     &    ' in kinematical channels:'
         iwfrst=0
        end if
        write(7,'(i7,a,i7,a,e10.3,a,e10.3)') i,':',iwkin(i),
     &  ' times, max discrepancy: dl0=',aw_w(i,2),' dl=',aw_w(i,3)
       end if
      end do

      close(7)

      call MPI_FINALIZE (ierr)

      end
